@@ -42,60 +42,62 @@
 ;;; along with this program; if not, write to the Free Software
 ;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(provide 'vm-menu)
+;;(provide 'vm-menu)
 
 (defvar vm-menu-folders-menu 
   '("Manipulate Folders"
     ["Make Folders Menu" vm-menu-hm-make-folder-menu vm-folder-directory])
   "VM folder menu list.")
 
-(defconst vm-menu-folder-menu
-  (list
-   "Folder"
-   (if vm-fsfemacs-p
-       ["Manipulate Folders" ignore (ignore)]
-     vm-menu-folders-menu)
-   "---"
-   ["Display Summary" vm-summarize t]
-   ["Toggle Threading" vm-toggle-threads-display t]
-   "---"
-   ["Get New Mail" vm-get-new-mail (vm-menu-can-get-new-mail-p)]
-   "---"
-   ["Search" vm-isearch-forward vm-message-list]
-   "---"
-   ["Auto-Archive" vm-auto-archive-messages vm-message-list]
-   ["Expunge" vm-expunge-folder vm-message-list]
-   ["Expunge POP Messages" vm-expunge-pop-messages t]
-   "---"
-   ["Visit Folder" vm-visit-folder t]
-   ["Visit POP Folder" vm-visit-pop-folder t]
-   ["Revert Folder (back to disk version)" revert-buffer (vm-menu-can-revert-p)]
-   ["Recover Folder (from auto-save file)" recover-file (vm-menu-can-recover-p)]
-   ["Save" vm-save-folder (vm-menu-can-save-p)]
-   ["Save As..." vm-write-file t]
-   ["Quit" vm-quit-no-change t]
-   ["Save & Quit" vm-quit t]
-   "---"
-   "---"
-   ;; special string that marks the tail of this menu for
-   ;; vm-menu-install-visited-folders-menu.
-   "-------"
-   ))
+(defvar vm-menu-folder-menu
+  `("Folder"
+    ,(if vm-fsfemacs-p
+	["Manipulate Folders" ignore (ignore)]
+      vm-menu-folders-menu)
+    "---"
+    ["Display Summary" vm-summarize t]
+    ["Toggle Threading" vm-toggle-threads-display t]
+    "---"
+    ["Get New Mail" vm-get-new-mail (vm-menu-can-get-new-mail-p)]
+    "---"
+    ["Search" vm-isearch-forward vm-message-list]
+    "---"
+    ["Auto-Archive" vm-auto-archive-messages vm-message-list]
+    ["Expunge" vm-expunge-folder vm-message-list]
+    ["Expunge POP Messages" vm-expunge-pop-messages
+     (vm-menu-can-expunge-pop-messages-p)]
+    "---"
+    ["Visit Folder" vm-visit-folder t]
+    ["Visit POP Folder" vm-visit-pop-folder t]
+    ["Revert Folder (back to disk version)" vm-revert-buffer
+     (vm-menu-can-revert-p)]
+    ["Recover Folder (from auto-save file)" vm-recover-file
+     (vm-menu-can-recover-p)]
+    ["Save" vm-save-folder (vm-menu-can-save-p)]
+    ["Save As..." vm-write-file t]
+    ["Quit" vm-quit-no-change t]
+    ["Save & Quit" vm-quit t]
+    "---"
+    "---"
+    ;; special string that marks the tail of this menu for
+    ;; vm-menu-install-visited-folders-menu.
+    "-------"
+    ))
 
-(defconst vm-menu-dispose-menu
+(defvar vm-menu-dispose-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Dispose"
 			 "Dispose"
 			 "---"
 			 "---")
 		 (list "Dispose"))))
-    (append
-     title
-     (list
+    `(,@title
       ["Reply to Author" vm-reply vm-message-list]
       ["Reply to All" vm-followup vm-message-list]
-      ["Reply to Author (citing original)" vm-reply-include-text vm-message-list]
-      ["Reply to All (citing original)" vm-followup-include-text vm-message-list]
+      ["Reply to Author (citing original)" vm-reply-include-text
+       vm-message-list]
+      ["Reply to All (citing original)" vm-followup-include-text
+       vm-message-list]
       ["Forward" vm-forward-message vm-message-list]
       ["Resend" vm-resend-message vm-message-list]
       ["Retry Bounce" vm-resend-bounced-message vm-message-list]
@@ -111,9 +113,9 @@
       "---"
       ["Burst Message as Digest" (vm-burst-digest "guess") vm-message-list]
       ["Decode MIME" vm-decode-mime-message (vm-menu-can-decode-mime-p)]
-      ))))
+      )))
 
-(defconst vm-menu-motion-menu
+(defvar vm-menu-motion-menu
   '("Motion"
     ["Page Up" vm-scroll-backward vm-message-list]
     ["Page Down" vm-scroll-forward vm-message-list]
@@ -141,7 +143,7 @@
     ["Go to Parent Message" vm-goto-parent-message t]
     ))
 
-(defconst vm-menu-virtual-menu
+(defvar vm-menu-virtual-menu
   '("Virtual"
     ["Visit Virtual Folder" vm-visit-virtual-folder t]
     ["Create Virtual Folder" vm-create-virtual-folder t]
@@ -153,7 +155,7 @@
     "-------"
     ))
 
-(defconst vm-menu-send-menu
+(defvar vm-menu-send-menu
   '("Send"
     ["Compose" vm-mail t]
     ["Continue Composing" vm-continue-composing-message vm-message-list]
@@ -169,7 +171,7 @@
     ["Send MIME Digest" vm-send-mime-digest vm-message-list]
     ))
 
-(defconst vm-menu-mark-menu
+(defvar vm-menu-mark-menu
   '("Mark"
     ["Next Command Uses Marks..." vm-next-command-uses-marks
      :active vm-message-list
@@ -193,14 +195,14 @@
     ["Unmark Thread Subtree" vm-unmark-thread-subtree vm-message-list]
     ))
 
-(defconst vm-menu-label-menu
+(defvar vm-menu-label-menu
   '("Label"
     ["Add Label" vm-add-message-labels vm-message-list]
     ["Add Existing Label" vm-add-existing-message-labels vm-message-list]
     ["Remove Label" vm-delete-message-labels vm-message-list]
     ))
 
-(defconst vm-menu-sort-menu
+(defvar vm-menu-sort-menu
   '("Sort"
     ["By Multiple Fields..." vm-sort-messages vm-message-list]
     "---"
@@ -223,7 +225,7 @@
     ["Revert to Physical Order" (vm-sort-messages "physical-order" t) vm-message-list]
     ))
 
-(defconst vm-menu-help-menu
+(defvar vm-menu-help-menu
   '("Help!"
     ["What Now?" vm-help t]
     ["Describe Mode" describe-mode t]
@@ -234,189 +236,186 @@
     ["Quit Without Saving" vm-quit-no-change t]
     ))
 
-(defconst vm-menu-undo-menu
+(defvar vm-menu-undo-menu
   ["Undo" vm-undo (vm-menu-can-undo-p)]
   )
 
-(defconst vm-menu-emacs-button
+(defvar vm-menu-emacs-button
   ["XEmacs" vm-menu-toggle-menubar t]
   )
 
-(defconst vm-menu-vm-button
+(defvar vm-menu-vm-button
   ["VM" vm-menu-toggle-menubar t]
   )
 
-(defconst vm-menu-mail-menu
+(defvar vm-menu-mail-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Mail Commands"
 			 "Mail Commands"
 			 "---"
 			 "---")
 		 (list "Mail Commands"))))
-    (append
-     title
-     (list ["Send and Exit" vm-mail-send-and-exit (vm-menu-can-send-mail-p)]
-	   ["Send, Keep Composing" vm-mail-send (vm-menu-can-send-mail-p)]
-	   ["Cancel" kill-buffer t]
-	   "----"
-	   ["Yank Original" vm-menu-yank-original vm-reply-list]
-	   "----"
-	   (append
-	    (if (vm-menu-fsfemacs19-menus-p)
-		(list "Send Using MIME..."
-		      "Send Using MIME..."
-		      "---"
-		      "---")
-	      (list "Send Using MIME..."))
-	    (list
-	     ["Use MIME"
-	      (progn (set (make-local-variable 'vm-send-using-mime) t)
-		     (vm-mail-mode-remove-tm-hooks))
-	      :active t
-	      :style radio
-	      :selected vm-send-using-mime]
-	     ["Don't use MIME"
-	      (set (make-local-variable 'vm-send-using-mime) nil)
-	      :active t
-	      :style radio
-	      :selected (not vm-send-using-mime)]))
-	   (append
-	    (if (vm-menu-fsfemacs19-menus-p)
-		(list "Fragment Messages Larger Than ..."
-		      "Fragment Messages Larger Than ..."
-		      "---"
-		      "---")
-	      (list "Fragment Messages Larger Than ..."))
-	    (list ["Infinity, i.e., don't fragment"
-		   (set (make-local-variable 'vm-mime-max-message-size) nil)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size nil)]
-		  ["50000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			50000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 50000)]
-		  ["100000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			100000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 100000)]
-		  ["200000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			200000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 200000)]
-		  ["500000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			500000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 500000)]
-		  ["1000000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			1000000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 1000000)]
-		  ["2000000 bytes"
-		   (set (make-local-variable 'vm-mime-max-message-size)
-			2000000)
-		   :active vm-send-using-mime
-		   :style radio
-		   :selected (eq vm-mime-max-message-size 2000000)]))
-	   (append
-	    (if (vm-menu-fsfemacs19-menus-p)
-		(list "Encode 8-bit Characters Using ..."
-		      "Encode 8-bit Characters Using ..."
-		      "---"
-		      "---")
-	      (list "Encode 8-bit Characters Using ..."))
-	    (list
-	     ["Nothing, i.e., send unencoded"
-	      (set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
-		   '8bit)
-	      :active vm-send-using-mime
-	      :style radio
-	      :selected (eq vm-mime-8bit-text-transfer-encoding '8bit)]
-	     ["Quoted-Printable"
-	      (set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
-		   'quoted-printable)
-	      :active vm-send-using-mime
-	      :style radio
-	      :selected (eq vm-mime-8bit-text-transfer-encoding
-			    'quoted-printable)]
-	     ["BASE64"
-	      (set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
-		   'base64)
-	      :active vm-send-using-mime
-	      :style radio
-	      :selected (eq vm-mime-8bit-text-transfer-encoding 'base64)]))
-	   "----"
-	   ["Attach File..."	vm-mime-attach-file vm-send-using-mime]
+    `(,@title
+      ["Send and Exit" vm-mail-send-and-exit (vm-menu-can-send-mail-p)]
+      ["Send, Keep Composing" vm-mail-send (vm-menu-can-send-mail-p)]
+      ["Cancel" kill-buffer t]
+      "----"
+      ["Yank Original" vm-menu-yank-original vm-reply-list]
+      "----"
+      (
+       ,@(if (vm-menu-fsfemacs19-menus-p)
+	     (list "Send Using MIME..."
+		   "Send Using MIME..."
+		   "---"
+		   "---")
+	   (list "Send Using MIME..."))
+       ["Use MIME"
+	(progn (set (make-local-variable 'vm-send-using-mime) t)
+	       (vm-mail-mode-remove-tm-hooks))
+	:active t
+	:style radio
+	:selected vm-send-using-mime]
+       ["Don't use MIME"
+	(set (make-local-variable 'vm-send-using-mime) nil)
+	:active t
+	:style radio
+	:selected (not vm-send-using-mime)])
+      (
+       ,@(if (vm-menu-fsfemacs19-menus-p)
+	     (list "Fragment Messages Larger Than ..."
+		   "Fragment Messages Larger Than ..."
+		   "---"
+		   "---")
+	   (list "Fragment Messages Larger Than ..."))
+       ["Infinity, i.e., don't fragment"
+	(set (make-local-variable 'vm-mime-max-message-size) nil)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size nil)]
+       ["50000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     50000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 50000)]
+       ["100000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     100000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 100000)]
+       ["200000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     200000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 200000)]
+       ["500000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     500000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 500000)]
+       ["1000000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     1000000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 1000000)]
+       ["2000000 bytes"
+	(set (make-local-variable 'vm-mime-max-message-size)
+	     2000000)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-max-message-size 2000000)])
+      (
+       ,@(if (vm-menu-fsfemacs19-menus-p)
+	     (list "Encode 8-bit Characters Using ..."
+		   "Encode 8-bit Characters Using ..."
+		   "---"
+		   "---")
+	   (list "Encode 8-bit Characters Using ..."))
+       ["Nothing, i.e., send unencoded"
+	(set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
+	     '8bit)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-8bit-text-transfer-encoding '8bit)]
+       ["Quoted-Printable"
+	(set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
+	     'quoted-printable)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-8bit-text-transfer-encoding
+		      'quoted-printable)]
+       ["BASE64"
+	(set (make-local-variable 'vm-mime-8bit-text-transfer-encoding)
+	     'base64)
+	:active vm-send-using-mime
+	:style radio
+	:selected (eq vm-mime-8bit-text-transfer-encoding 'base64)])
+      "----"
+      ["Attach File..."	vm-mime-attach-file vm-send-using-mime]
 ;;	   ["Attach MIME Message..." vm-mime-attach-mime-file
 ;;	    vm-send-using-mime]
-	   ["Encode MIME, But Don't Send" vm-mime-encode-composition
-	    (and vm-send-using-mime
-		 (null (vm-mail-mode-get-header-contents "MIME-Version:")))]
-	   ["Preview MIME Before Sending" vm-mime-preview-composition
-	    vm-send-using-mime]
-	   ))))
+      ["Encode MIME, But Don't Send" vm-mime-encode-composition
+       (and vm-send-using-mime
+	    (null (vm-mail-mode-get-header-contents "MIME-Version:")))]
+      ["Preview MIME Before Sending" vm-mime-preview-composition
+       vm-send-using-mime]
+      )))
 
-(defconst vm-menu-mime-dispose-menu
+(defvar vm-menu-mime-dispose-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on MIME body ..."
 			 "Take Action on MIME body ..."
 			 "---"
 			 "---")
 		 (list "Take Action on MIME body ..."))))
-    (append
-     title
-     (list ["Display as Text using Default Face"
-	    (vm-mime-run-display-function-at-point
-	     'vm-mime-display-body-as-text) t]
-	   ["Display using External Viewer"
-	    (vm-mime-run-display-function-at-point
-	     'vm-mime-display-body-using-external-viewer) t])
-     ;; FSF Emacs does not allow a non-string menu element name.
-     (if (vm-menu-can-eval-item-name)
-	 (list [(format "Convert to %s and Display"
-			(or (nth 1 (vm-mime-can-convert
-				(car
-				 (vm-mm-layout-type
-				  (vm-mime-get-button-layout e)))))
-			    "different type"))
-		(vm-mime-run-display-function-at-point
-		 'vm-mime-convert-body-then-display)
-		(vm-mime-can-convert (car (vm-mm-layout-type
-					   (vm-mime-get-button-layout e))))]))
-     (list "---"
-	   ["Save to File" vm-mime-reader-map-save-file t]
-	   ["Save to Folder" vm-mime-reader-map-save-message
-	    (let ((layout (vm-mime-run-display-function-at-point
-			   (function
-			    (lambda (e)
-			      (vm-extent-property e 'vm-mime-layout))))))
-	      (if (null layout)
-		  nil
-		(or (vm-mime-types-match "message/rfc822"
-					 (car (vm-mm-layout-type layout)))
-		    (vm-mime-types-match "message/news"
-					 (car (vm-mm-layout-type layout))))))]
-	   ["Send to Printer" (vm-mime-run-display-function-at-point
-			       'vm-mime-send-body-to-printer) t]
-	   ["Feed to Shell Pipeline (display output)"
-	    (vm-mime-run-display-function-at-point
-	     'vm-mime-pipe-body-to-queried-command) t]
-	   ["Feed to Shell Pipeline (discard output)"
-	    (vm-mime-run-display-function-at-point
-	     'vm-mime-pipe-body-to-queried-command-discard-output) t]
-	   ["Delete object" vm-delete-mime-object t]))))
+    `(,@title
+      ["Display as Text using Default Face"
+       (vm-mime-run-display-function-at-point
+	'vm-mime-display-body-as-text) t]
+      ["Display using External Viewer"
+       (vm-mime-run-display-function-at-point
+	'vm-mime-display-body-using-external-viewer) t]
+      ;; FSF Emacs does not allow a non-string menu element name.
+      ,@(if (vm-menu-can-eval-item-name)
+	    (list [(format "Convert to %s and Display"
+			   (or (nth 1 (vm-mime-can-convert
+				       (car
+					(vm-mm-layout-type
+					 (vm-mime-get-button-layout e)))))
+			       "different type"))
+		   (vm-mime-run-display-function-at-point
+		    'vm-mime-convert-body-then-display)
+		   (vm-mime-can-convert
+		    (car (vm-mm-layout-type
+			  (vm-mime-get-button-layout e))))]))
+      "---"
+      ["Save to File" vm-mime-reader-map-save-file t]
+      ["Save to Folder" vm-mime-reader-map-save-message
+       (let ((layout (vm-mime-run-display-function-at-point
+		      (function
+		       (lambda (e)
+			 (vm-extent-property e 'vm-mime-layout))))))
+	 (if (null layout)
+	     nil
+	   (or (vm-mime-types-match "message/rfc822"
+				    (car (vm-mm-layout-type layout)))
+	       (vm-mime-types-match "message/news"
+				    (car (vm-mm-layout-type layout))))))]
+      ["Send to Printer" (vm-mime-run-display-function-at-point
+			  'vm-mime-send-body-to-printer) t]
+      ["Feed to Shell Pipeline (display output)"
+       (vm-mime-run-display-function-at-point
+	'vm-mime-pipe-body-to-queried-command) t]
+      ["Feed to Shell Pipeline (discard output)"
+       (vm-mime-run-display-function-at-point
+	'vm-mime-pipe-body-to-queried-command-discard-output) t]
+      ["Delete object" vm-delete-mime-object t])))
 
-(defconst vm-menu-url-browser-menu
+(defvar vm-menu-url-browser-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Send URL to ..."
 			 "Send URL to ..."
@@ -428,47 +427,48 @@
 		  ((fboundp 'w3-fetch)
 		   'w3-fetch)
 		  (t 'w3-fetch-other-frame))))
-    (append
-     title
-     (list (vector "Emacs W3"
-		   (list 'vm-mouse-send-url-at-position
-			 '(point)
-			 (list 'quote w3))
-		   (list 'fboundp (list 'quote w3)))
-	   ["Mosaic"
-	    (vm-mouse-send-url-at-position (point)
-					   'vm-mouse-send-url-to-mosaic)
-	    t]
-	   ["mMosaic"
-	    (vm-mouse-send-url-at-position (point)
-					   'vm-mouse-send-url-to-mmosaic)
-	    t]
-	   ["Netscape"
-	    (vm-mouse-send-url-at-position (point)
-					   'vm-mouse-send-url-to-netscape)
-	    t]))))
+    `(,@title
+      ["Emacs W3" (vm-mouse-send-url-at-position (point) (quote ,w3))
+       (fboundp (quote ,w3))]
+      ["Mosaic"
+       (vm-mouse-send-url-at-position (point)
+				      'vm-mouse-send-url-to-mosaic)
+       t]
+      ["mMosaic"
+       (vm-mouse-send-url-at-position (point)
+				      'vm-mouse-send-url-to-mmosaic)
+       t]
+      ["Netscape"
+       (vm-mouse-send-url-at-position (point)
+				      'vm-mouse-send-url-to-netscape)
+       t]
+      ["Konqueror"
+       (vm-mouse-send-url-at-position (point)
+				      'vm-mouse-send-url-to-konqueror)
+       t]
+      ["X Clipboard"
+       (vm-mouse-send-url-at-position (point)
+				      'vm-mouse-send-url-to-clipboard)
+       t])))
 
-(defconst vm-menu-mailto-url-browser-menu
+(defvar vm-menu-mailto-url-browser-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Send Mail using ..."
 			 "Send Mail using ..."
 			 "---"
 			 "---")
 		 (list "Send Mail using ..."))))
-    (append
-     title
-     (list ["VM" (vm-mouse-send-url-at-position (point) 'ignore) t]))))
+    `(,@title
+      ["VM" (vm-mouse-send-url-at-position (point) 'ignore) t])))
 
-(defconst vm-menu-subject-menu
+(defvar vm-menu-subject-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on Subject..."
 			 "Take Action on Subject..."
 			 "---"
 			 "---")
 		 (list "Take Action on Subject..."))))
-    (append
-     title
-     (list
+    `(,@title
       ["Kill Subject" vm-kill-subject vm-message-list]
       ["Next Message, Same Subject" vm-next-message-same-subject
        vm-message-list]
@@ -480,75 +480,137 @@
        vm-message-list]
       ["Virtual Folder, Matching Subject" vm-menu-create-subject-virtual-folder
        vm-message-list]
-      ))))
+      )))
 
-(defconst vm-menu-author-menu
+(defvar vm-menu-author-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "Take Action on Author..."
 			 "Take Action on Author..."
 			 "---"
 			 "---")
 		 (list "Take Action on Author..."))))
-    (append
-     title
-     (list
+    `(,@title
       ["Mark Messages, Same Author" vm-mark-messages-same-author
        vm-message-list]
       ["Unmark Messages, Same Author" vm-unmark-messages-same-author
        vm-message-list]
       ["Virtual Folder, Matching Author" vm-menu-create-author-virtual-folder
        vm-message-list]
-      ))))
+      )))
 
-(defconst vm-menu-content-disposition-menu
+(defvar vm-menu-attachment-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
-		   (list "Set Content Disposition"
-			 "Set Content Disposition"
+		   (list "Fiddle With Attachment"
+			 "Fiddle With Attachment"
 			 "---"
 			 "---")
-		 (list "Set Content Disposition"))))
-    (append
-     title
-     (list ["Unspecified"
-	    (vm-mime-set-attachment-disposition-at-point 'unspecified)
-	    :active vm-send-using-mime
-	    :style radio
-	    :selected (eq (vm-mime-attachment-disposition-at-point)
-			  'unspecified)]
-	   ["Inline"
-	    (vm-mime-set-attachment-disposition-at-point 'inline)
-	    :active vm-send-using-mime
-	    :style radio
-	    :selected (eq (vm-mime-attachment-disposition-at-point) 'inline)]
-	   ["Attachment"
-	    (vm-mime-set-attachment-disposition-at-point 'attachment)
-	    :active vm-send-using-mime
-	    :style radio
-	    :selected (eq (vm-mime-attachment-disposition-at-point)
-			  'attachment)]))))
+		 (list "Fiddle With Attachment"))))
+    `(,@title
+      (
+       ,@(if (vm-menu-fsfemacs19-menus-p)
+	     (list "Set Content Disposition..."
+		   "Set Content Disposition..."
+		   "---"
+		   "---")
+	   (list "Set Content Disposition..."))
+	 ["Unspecified"
+	  (vm-mime-set-attachment-disposition-at-point 'unspecified)
+	  :active vm-send-using-mime
+	  :style radio
+	  :selected (eq (vm-mime-attachment-disposition-at-point)
+			'unspecified)]
+	 ["Inline"
+	  (vm-mime-set-attachment-disposition-at-point 'inline)
+	  :active vm-send-using-mime
+	  :style radio
+	  :selected (eq (vm-mime-attachment-disposition-at-point) 'inline)]
+	 ["Attachment"
+	  (vm-mime-set-attachment-disposition-at-point 'attachment)
+	  :active vm-send-using-mime
+	  :style radio
+	  :selected (eq (vm-mime-attachment-disposition-at-point)
+			'attachment)])
+      (
+       ,@(if (vm-menu-fsfemacs19-menus-p)
+	     (list "Forward Local External Bodies"
+		   "Forward Local External Bodies"
+		   "---"
+		   "---")
+	   (list "Forward Local External Bodies"))
+	 ["Forward Unchanged"
+	  (vm-mime-set-attachment-forward-local-refs-at-point t)
+	  :active vm-send-using-mime
+	  :style radio
+	  :selected (vm-mime-attachment-forward-local-refs-at-point)]
+	 ["Convert to Internal Object"
+	  (vm-mime-set-attachment-forward-local-refs-at-point nil)
+	  :active vm-send-using-mime
+	  :style radio
+	  :selected (not (vm-mime-attachment-forward-local-refs-at-point))])
+      )))
+
+(defvar vm-menu-image-menu
+  (let ((title (if (vm-menu-fsfemacs19-menus-p)
+		   (list "Redisplay Image"
+			 "Redisplay Image"
+			 "---"
+			 "---")
+		 (list "Redisplay Image"))))
+    `(,@title
+      ["4x Larger"
+       (vm-mime-run-display-function-at-point 'vm-mime-larger-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["4x Smaller"
+       (vm-mime-run-display-function-at-point 'vm-mime-smaller-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["Rotate Left"
+       (vm-mime-run-display-function-at-point 'vm-mime-rotate-image-left)
+       (stringp vm-imagemagick-convert-program)]
+      ["Rotate Right"
+       (vm-mime-run-display-function-at-point 'vm-mime-rotate-image-right)
+       (stringp vm-imagemagick-convert-program)]
+      ["Mirror"
+       (vm-mime-run-display-function-at-point 'vm-mime-mirror-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["Brighter"
+       (vm-mime-run-display-function-at-point 'vm-mime-brighten-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["Dimmer"
+       (vm-mime-run-display-function-at-point 'vm-mime-dim-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["Monochrome"
+       (vm-mime-run-display-function-at-point 'vm-mime-monochrome-image)
+       (stringp vm-imagemagick-convert-program)]
+      ["Revert to Original"
+       (vm-mime-run-display-function-at-point 'vm-mime-revert-image)
+       (get
+	(vm-mm-layout-cache
+	 (vm-extent-property (vm-find-layout-extent-at-point) 'vm-mime-layout))
+	'vm-image-modified)]
+      )))
 
 (defvar vm-menu-vm-menubar nil)
 
-(defconst vm-menu-vm-menu
+(defvar vm-menu-vm-menu
   (let ((title (if (vm-menu-fsfemacs19-menus-p)
 		   (list "VM"
 			 "VM"
 			 "---"
 			 "---")
 		 (list "VM"))))
-    (append title
-	    (list vm-menu-folder-menu
-		  vm-menu-motion-menu
-		  vm-menu-send-menu
-		  vm-menu-mark-menu
-		  vm-menu-label-menu
-		  vm-menu-sort-menu
-		  vm-menu-virtual-menu
-;;		  vm-menu-undo-menu
-		  vm-menu-dispose-menu
-		  "---"
-		  "---"
-		  vm-menu-help-menu))))
+    `(,@title
+      ,vm-menu-folder-menu
+      ,vm-menu-motion-menu
+      ,vm-menu-send-menu
+      ,vm-menu-mark-menu
+      ,vm-menu-label-menu
+      ,vm-menu-sort-menu
+      ,vm-menu-virtual-menu
+;;    ,vm-menu-undo-menu
+      ,vm-menu-dispose-menu
+      "---"
+      "---"
+      ,vm-menu-help-menu)))
 
 (defvar vm-mode-menu-map nil)
 
@@ -611,6 +673,13 @@ set to the command name so that window configuration will be done."
 	     (not (vm-mime-plain-message-p (car vm-message-pointer)))))
     (error nil)))
 
+(defun vm-menu-can-expunge-pop-messages-p ()
+  (condition-case nil
+      (save-excursion
+	(vm-select-folder-buffer)
+	(not (eq vm-folder-access-method 'pop)))
+    (error nil)))
+
 (defun vm-menu-yank-original ()
   (interactive)
   (save-excursion
@@ -707,10 +776,14 @@ set to the command name so that window configuration will be done."
 	(vm-easy-menu-define vm-menu-fsfemacs-mime-dispose-menu
 			     (list dummy) nil
 			     vm-menu-mime-dispose-menu)
-	;; content disposition menu
-	(vm-easy-menu-define vm-menu-fsfemacs-content-disposition-menu
+	;; attachment menu
+	(vm-easy-menu-define vm-menu-fsfemacs-attachment-menu
 			     (list dummy) nil
-			     vm-menu-content-disposition-menu)
+			     vm-menu-attachment-menu)
+	;; image menu
+	(vm-easy-menu-define vm-menu-fsfemacs-image-menu
+			     (list dummy) nil
+			     vm-menu-image-menu)
 	;; block the global menubar entries in the map so that VM
 	;; can take over the menubar if necessary.
 	(define-key map [rootmenu] (make-sparse-keymap))
@@ -808,7 +881,7 @@ set to the command name so that window configuration will be done."
 	 (goto-char (posn-point (event-start event)))
 	 (vm-menu-popup-fsfemacs-menu event))))
 
-(defvar vm-menu-fsfemacs-content-disposition-menu)
+(defvar vm-menu-fsfemacs-attachment-menu)
 (defun vm-menu-popup-context-menu (event)
   (interactive "e")
   ;; We should not need to do anything here for XEmacs.  The
@@ -823,7 +896,7 @@ set to the command name so that window configuration will be done."
 	 (goto-char (posn-point (event-start event)))
 	 (if (get-text-property (point) 'vm-mime-object)
 	     (vm-menu-popup-fsfemacs-menu
-	      event vm-menu-fsfemacs-content-disposition-menu)
+	      event vm-menu-fsfemacs-attachment-menu)
 	   (let (o-list o menu (found nil))
 	     (setq o-list (overlays-at (point)))
 	     (while (and o-list (not found))
@@ -833,6 +906,9 @@ set to the command name so that window configuration will be done."
 		     ((setq menu (overlay-get (car o-list) 'vm-header))
 		      (setq found t)
 		      (vm-menu-popup-fsfemacs-menu event menu))
+		     ((setq menu (overlay-get (car o-list) 'vm-image))
+		      (setq found t)
+		      (vm-menu-popup-fsfemacs-menu event menu))
 		     ((overlay-get (car o-list) 'vm-mime-layout)
 		      (setq found t)
 		      (vm-menu-popup-mime-dispose-menu event)))
@@ -884,14 +960,24 @@ set to the command name so that window configuration will be done."
 	 (vm-menu-popup-fsfemacs-menu
 	  event vm-menu-fsfemacs-mime-dispose-menu))))
 
-(defun vm-menu-popup-content-disposition-menu (event)
+(defun vm-menu-popup-attachment-menu (event)
   (interactive "e")
   (vm-menu-goto-event event)
   (cond ((and (vm-menu-xemacs-menus-p) vm-use-menus)
-	 (popup-menu vm-menu-content-disposition-menu))
+	 (popup-menu vm-menu-attachment-menu))
 	((and (vm-menu-fsfemacs-menus-p) vm-use-menus)
 	 (vm-menu-popup-fsfemacs-menu
-	  event vm-menu-fsfemacs-content-disposition-menu))))
+	  event vm-menu-fsfemacs-attachment-menu))))
+
+(defvar vm-menu-fsfemacs-image-menu)
+(defun vm-menu-popup-image-menu (event)
+  (interactive "e")
+  (vm-menu-goto-event event)
+  (cond ((and (vm-menu-xemacs-menus-p) vm-use-menus)
+	 (popup-menu vm-menu-image-menu))
+	((and (vm-menu-fsfemacs-menus-p) vm-use-menus)
+	 (vm-menu-popup-fsfemacs-menu
+	  event vm-menu-fsfemacs-image-menu))))
 
 ;; to quiet the byte-compiler
 (defvar vm-menu-fsfemacs-mail-menu)
@@ -1428,3 +1514,5 @@ for the current directory (.) is inserted."
     menulist
     )
   )
+
+(provide 'vm-menu)
